@@ -460,6 +460,7 @@ tree_t make_ref(tree_t to)
    tree_set_ident(t, tree_ident(to));
    tree_set_ref(t, to);
    tree_set_type(t, tree_type(to));
+   tree_set_loc(t, tree_loc(to));
    return t;
 }
 
@@ -1149,6 +1150,9 @@ void intern_strings(void)
    id_cache[W_VITAL]           = ident_new("VITAL");
    id_cache[W_RESOLUTION]      = ident_new("resolution");
    id_cache[W_TEXT_UTIL]       = ident_new("NVC.TEXT_UTIL");
+   id_cache[W_VERILOG_LOGIC]   = ident_new("NVC.VERILOG.T_LOGIC");
+   id_cache[W_DLR_SIGNED]      = ident_new("$signed");
+   id_cache[W_DLR_CLOG2]       = ident_new("$clog2");
 
    id_cache[W_IEEE_LOGIC_VECTOR] =
       ident_new("IEEE.STD_LOGIC_1164.STD_LOGIC_VECTOR");
@@ -1161,6 +1165,8 @@ void intern_strings(void)
 
    id_cache[W_NUMERIC_STD_UNSIGNED] = ident_new("IEEE.NUMERIC_STD_UNSIGNED");
    id_cache[W_NUMERIC_BIT_UNSIGNED] = ident_new("IEEE.NUMERIC_BIT_UNSIGNED");
+   id_cache[W_VERILOG_NET_VALUE]    = ident_new("NVC.VERILOG.T_NET_VALUE");
+   id_cache[W_VERILOG_WIRE_ARRAY]   = ident_new("NVC.VERILOG.T_WIRE_ARRAY");
 
    id_cache[W_OP_CCONV]               = ident_new("\"??\"");
    id_cache[W_OP_AND]                 = ident_new("\"and\"");
@@ -2598,7 +2604,8 @@ void analyse_file(const char *file, jit_t *jit, unit_registry_t *ur,
          LOCAL_TEXT_BUF tb = tb_new();
          vlog_preprocess(tb, true);
 
-         input_from_buffer(tb_get(tb), tb_len(tb), SOURCE_VERILOG);
+         file_ref_t file_ref = loc_file_ref(file, NULL);
+         input_from_buffer(tb_get(tb), tb_len(tb), file_ref, SOURCE_VERILOG);
 
          lib_t work = lib_work();
          vlog_node_t module;
@@ -2956,3 +2963,62 @@ bool can_be_signal(type_t type)
       return true;
    }
 }
+
+type_t merge_constraints(type_t to, type_t from)
+{
+   assert(type_is_unconstrained(to));
+   assert(type_eq(to, from));
+
+   tree_t cto = NULL;
+   if (type_kind(to) == T_SUBTYPE && type_has_constraint(to))
+      cto = type_constraint(to);
+
+   tree_t cfrom = NULL;
+   if (type_kind(from) == T_SUBTYPE && type_has_constraint(from))
+      cfrom = type_constraint(from);
+
+   if (cfrom == NULL)
+      return to;
+
+   type_t sub = type_new(T_SUBTYPE);
+   type_set_base(sub, type_base_recur(to));
+
+   if (type_is_array(to)) {
+      type_set_constraint(sub, cto ?: cfrom);
+
+      type_t elem = type_elem(to);
+      if (type_is_unconstrained(elem))
+         type_set_elem(sub, merge_constraints(elem, type_elem(from)));
+      else
+         type_set_elem(sub, elem);
+   }
+   else {
+      tree_t cnew = tree_new(T_CONSTRAINT);
+      tree_set_subkind(cnew, C_RECORD);
+      tree_set_loc(cnew, tree_loc(cto ?: cfrom));
+
+      type_set_constraint(sub, cnew);
+
+      if (cto != NULL) {
+         const int nto = tree_ranges(cto);
+         for (int i = 0; i < nto; i++)
+            tree_add_range(cnew, tree_range(cto, i));
+      }
+
+      const int nfrom = tree_ranges(cfrom), base = tree_ranges(cnew);
+      for (int i = 0; i < nfrom; i++) {
+         tree_t ec = tree_range(cfrom, i);
+         assert(tree_kind(ec) == T_ELEM_CONSTRAINT);
+
+         ident_t id = tree_ident(ec);
+         bool found = false;
+         for (int j = 0; j < base && !found; j++)
+            found |= tree_ident(tree_range(cnew, j)) == id;
+
+         if (!found)
+            tree_add_range(cnew, ec);
+      }
+   }
+
+   return sub;
+}
