@@ -1,5 +1,5 @@
 //
-//  Copyright (C) 2013-2024  Nick Gasson
+//  Copyright (C) 2013-2025  Nick Gasson
 //
 //  This program is free software: you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,6 +19,9 @@
 #include "array.h"
 #include "cov/cov-api.h"
 #include "cov/cov-data.h"
+#include "cov/cov-priv.h"
+#include "cov/cov-structs.h"
+#include "hash.h"
 #include "ident.h"
 #include "lib.h"
 #include "object.h"
@@ -43,6 +46,7 @@ typedef enum {
    CTRL_PUSH_SCOPE,
    CTRL_POP_SCOPE,
    CTRL_END_OF_FILE,
+   CTRL_PUSH_UNIT,
 } cov_control_t;
 
 static const struct {
@@ -61,7 +65,13 @@ static const struct {
 };
 
 #define COVER_FILE_MAGIC   0x6e636462   // ASCII "ncdb"
-#define COVER_FILE_VERSION 2
+#define COVER_FILE_VERSION 4
+
+static inline unsigned get_next_tag(cover_block_t *b)
+{
+   assert(b->data == NULL);
+   return b->next_tag++;
+}
 
 static bool cover_is_branch(tree_t branch)
 {
@@ -188,7 +198,7 @@ static int32_t cover_add_item(cover_data_t *data, cover_scope_t *cs,
 
    cover_item_t new = {
       .kind          = kind,
-      .tag           = data->next_tag++,
+      .tag           = get_next_tag(cs->block),
       .data          = 0,
       .flags         = flags,
       .loc           = loc,
@@ -641,18 +651,22 @@ cover_item_t *cover_add_items_for(cover_data_t *data, cover_scope_t *cs,
    if (!cs->emit)
       return NULL;
 
-   // Skip coverage emit when ignored by spec-file -> Common for all item kinds
-   cover_scope_t *ignore_scope = cs;
-   for (; ignore_scope->type != CSCOPE_INSTANCE && ignore_scope->parent;
-        ignore_scope = ignore_scope->parent)
-      ;
-
    const loc_t loc = get_cover_loc(kind, obj);
 
-   for (int i = 0; i < ignore_scope->ignore_lines.count; i++) {
-      line_range_t *lr = &(ignore_scope->ignore_lines.items[i]);
-      if (loc.first_line > lr->start && loc.first_line <= lr->end)
-         return NULL;
+   // Multiple scopes may be emitted from a single file for generate
+   // statements, blocks, etc.
+   for (cover_scope_t *ignore_scope = cs; ignore_scope->parent;
+        ignore_scope = ignore_scope->parent) {
+      if (cover_block_kind(ignore_scope) == CBLOCK_NONE)
+         continue;
+      else if (ignore_scope->loc.file_ref != loc.file_ref)
+         break;
+
+      for (int i = 0; i < ignore_scope->ignore_lines.count; i++) {
+         line_range_t *lr = &(ignore_scope->ignore_lines.items[i]);
+         if (loc.first_line > lr->start && loc.first_line <= lr->end)
+            return NULL;
+      }
    }
 
    // Emit items based on item kind
@@ -749,32 +763,37 @@ void cover_merge_one_item(cover_item_t *item, int32_t data)
    }
 }
 
-static void cover_update_counts(cover_scope_t *s, const int32_t *counts)
+static void cover_update_counts(cover_scope_t *s)
 {
-   for (int i = 0; i < s->items.count; i++) {
-      cover_item_t *item = &(s->items.items[i]);
-
-      const int32_t data = counts ? counts[item->tag] : 0;
-      cover_merge_one_item(item, data);
+   if (s->block != NULL && s->block->data != NULL) {
+      for (int i = 0; i < s->items.count; i++) {
+         cover_item_t *item = &(s->items.items[i]);
+         cover_merge_one_item(item, s->block->data[item->tag]);
+      }
    }
 
    for (int i = 0; i < s->children.count; i++)
-      cover_update_counts(s->children.items[i], counts);
+      cover_update_counts(s->children.items[i]);
 }
 
 static void cover_write_scope(cover_scope_t *s, fbuf_t *f,
                               ident_wr_ctx_t ident_ctx, loc_wr_ctx_t *loc_ctx)
 {
-   write_u8(CTRL_PUSH_SCOPE, f);
+   if (s->block != NULL && s == s->block->self) {
+      write_u8(CTRL_PUSH_UNIT, f);
+
+      ident_write(s->block->name, ident_ctx);
+      ident_write(s->block->block_name, ident_ctx);
+      fbuf_put_uint(f, s->block->kind);
+      fbuf_put_uint(f, s->block->next_tag);
+   }
+   else
+      write_u8(CTRL_PUSH_SCOPE, f);
 
-   fbuf_put_uint(f, s->type);
    ident_write(s->name, ident_ctx);
    ident_write(s->hier, ident_ctx);
    loc_write(&s->loc, loc_ctx);
 
-   if (s->type == CSCOPE_INSTANCE)
-      ident_write(s->block_name, ident_ctx);
-
    fbuf_put_uint(f, s->items.count);
    for (int i = 0; i < s->items.count; i++) {
       cover_item_t *item = &(s->items.items[i]);
@@ -816,12 +835,10 @@ static void cover_write_scope(cover_scope_t *s, fbuf_t *f,
 LCOV_EXCL_START
 static void cover_debug_dump(cover_scope_t *s, int indent)
 {
-   color_printf("%*s$!blue$%s", indent, "", istr(s->name));
-   switch (s->type) {
-   case CSCOPE_INSTANCE: color_printf(" : instance"); break;
-   default: break;
-   }
-   color_printf("$$\n");
+   color_printf("%*s$!blue$%s$$", indent, "", istr(s->name));
+   if (cover_is_hier(s))
+      printf(" : %s", istr(s->block->block_name));
+   color_printf("\n");
 
    for (int i = 0; i < s->items.count; i++) {
       cover_item_t *item = &(s->items.items[i]);
@@ -837,11 +854,10 @@ static void cover_debug_dump(cover_scope_t *s, int indent)
 }
 LCOV_EXCL_STOP
 
-void cover_dump_items(cover_data_t *data, fbuf_t *f, cover_dump_t dt,
-                      const int32_t *counts)
+void cover_dump_items(cover_data_t *data, fbuf_t *f, cover_dump_t dt)
 {
    if (dt == COV_DUMP_RUNTIME)
-      cover_update_counts(data->root_scope, counts);
+      cover_update_counts(data->root_scope);
 
    if (opt_get_int(OPT_COVER_VERBOSE))
       cover_debug_dump(data->root_scope, 0);
@@ -851,7 +867,6 @@ void cover_dump_items(cover_data_t *data, fbuf_t *f, cover_dump_t dt,
 
    fbuf_put_uint(f, data->mask);
    fbuf_put_uint(f, data->array_limit);
-   fbuf_put_uint(f, data->next_tag);
 
    loc_wr_ctx_t *loc_wr = loc_write_begin(f);
    ident_wr_ctx_t ident_ctx = ident_write_begin(f);
@@ -870,68 +885,143 @@ cover_data_t *cover_data_init(cover_mask_t mask, int array_limit, int threshold)
    data->mask = mask;
    data->array_limit = array_limit;
    data->threshold = threshold;
+   data->blocks = hash_new(16);
+   data->pool = pool_new();
 
    return data;
 }
 
+void cover_data_free(cover_data_t *db)
+{
+#ifdef DEBUG
+   size_t alloc, npages;
+   pool_stats(db->pool, &alloc, &npages);
+   if (npages > 0)
+      debugf("coverage database allocated %zu bytes in %zu pages",
+             alloc, npages);
+#endif
+
+   hash_free(db->blocks);
+   pool_free(db->pool);
+   free(db);
+}
+
 bool cover_enabled(cover_data_t *data, cover_mask_t mask)
 {
    return data != NULL && (data->mask & mask);
 }
 
-static bool cover_should_emit_scope(cover_data_t *data, cover_scope_t *cs,
-                                    tree_t t)
+static bool cover_should_emit_scope(cover_data_t *data, cover_scope_t *cs)
 {
    cover_spec_t *spc = data->spec;
    if (spc == NULL)
       return true;
 
    // Block (entity, package instance or block) name
-   if (cs->block_name) {
-      ident_t ename = ident_until(cs->block_name, '-');
+   if (cs->block->block_name != NULL) {
+      ident_t ename = ident_until(cs->block->block_name, '-');
 
-      for (int i = 0; i < spc->block_exclude.count; i++)
-
-         if (ident_glob(ename, AGET(spc->block_exclude, i), -1)) {
-#ifdef COVER_DEBUG_EMIT
-            printf("Cover emit: False, block (Block: %s, Pattern: %s)\n",
-                   istr(cs->block_name), AGET(spc->block_exclude, i));
-#endif
+      for (int i = 0; i < spc->block_exclude.count; i++) {
+         if (ident_glob(ename, AGET(spc->block_exclude, i), -1))
             return false;
-         }
+      }
 
-      for (int i = 0; i < data->spec->block_include.count; i++)
-         if (ident_glob(ename, AGET(spc->block_include, i), -1)) {
-#ifdef COVER_DEBUG_EMIT
-            printf("Cover emit: True, block (Block: %s, Pattern: %s)\n",
-                   istr(cs->block_name), AGET(spc->block_include, i));
-#endif
+      for (int i = 0; i < data->spec->block_include.count; i++) {
+         if (ident_glob(ename, AGET(spc->block_include, i), -1))
             return true;
-         }
+      }
    }
 
    // Hierarchy
-   for (int i = 0; i < spc->hier_exclude.count; i++)
-      if (ident_glob(cs->hier, AGET(spc->hier_exclude, i), -1)) {
-#ifdef COVER_DEBUG_EMIT
-         printf("Cover emit: False, hierarchy (Hierarchy: %s, Pattern: %s)\n",
-                istr(cs->hier), AGET(spc->hier_exclude, i));
-#endif
+   for (int i = 0; i < spc->hier_exclude.count; i++) {
+      if (ident_glob(cs->hier, AGET(spc->hier_exclude, i), -1))
          return false;
-      }
+   }
 
-   for (int i = 0; i < spc->hier_include.count; i++)
-      if (ident_glob(cs->hier, AGET(spc->hier_include, i), -1)) {
-#ifdef COVER_DEBUG_EMIT
-         printf("Cover emit: True, hierarchy (Hierarchy: %s, Pattern: %s)\n",
-                istr(cs->hier), AGET(spc->hier_include, i));
-#endif
+   for (int i = 0; i < spc->hier_include.count; i++) {
+      if (ident_glob(cs->hier, AGET(spc->hier_include, i), -1))
          return true;
-      }
+   }
 
    return false;
 }
 
+cover_scope_t *cover_create_block(cover_data_t *data, ident_t qual,
+                                  cover_scope_t *parent, tree_t inst,
+                                  tree_t unit, ident_t name)
+{
+   if (data == NULL)
+      return NULL;
+
+   cover_block_t *b = pool_calloc(data->pool, sizeof(cover_block_t));
+   b->name = qual;
+
+   assert(hash_get(data->blocks, qual) == NULL);
+   hash_put(data->blocks, qual, b);
+
+   if (parent == NULL) {
+      assert(data->root_scope == NULL);
+
+      parent = data->root_scope = xcalloc(sizeof(cover_scope_t));
+      parent->name = parent->hier = lib_name(lib_work());
+   }
+
+   ident_t scope_name;
+   if (name != NULL)
+      scope_name = name;
+   else if (tree_kind(inst) == T_INERTIAL)   // Process without label
+      scope_name = ident_sprintf("_S%u", parent->stmt_label++);
+   else if (tree_has_ident(inst))
+      scope_name = tree_ident(inst);
+   else
+      scope_name = ident_sprintf("_S%u", parent->stmt_label++);
+
+   cover_scope_t *s = pool_calloc(data->pool, sizeof(cover_scope_t));
+   s->name    = scope_name;
+   s->parent  = parent;
+   s->block   = b;
+   s->sig_pos = parent->sig_pos;
+   s->loc     = *tree_loc(unit);
+   s->hier    = ident_prefix(parent->hier, scope_name, '.');
+   s->emit    = cover_should_emit_scope(data, s);
+
+   switch (tree_kind(inst)) {
+   case T_BLOCK:
+      if (name == NULL) {
+         b->kind = CBLOCK_INSTANCE;
+         b->block_name = ident_rfrom(tree_ident(unit), '.');
+      }
+      else
+         b->kind = CBLOCK_USER;   // XXX
+      break;
+   case T_PROCESS:
+   case T_INERTIAL:
+      b->kind = CBLOCK_PROCESS;
+      b->block_name = parent->block->block_name;
+      break;
+   case T_PROC_BODY:
+   case T_FUNC_BODY:
+      b->kind = CBLOCK_SUBPROG;
+      break;
+   case T_PSL_DIRECT:
+      b->kind = CBLOCK_PROPERTY;
+      break;
+   case T_PACK_INST:
+   case T_PACKAGE:
+   case T_PACK_BODY:
+      b->kind = CBLOCK_PACKAGE;
+      b->block_name = tree_ident(unit);
+      break;
+   default:
+      should_not_reach_here();
+   }
+
+   APUSH(parent->children, s);
+   b->self = s;
+
+   return b->self;
+}
+
 cover_scope_t *cover_create_scope(cover_data_t *data, cover_scope_t *parent,
                                   tree_t t, ident_t name)
 {
@@ -941,7 +1031,7 @@ cover_scope_t *cover_create_scope(cover_data_t *data, cover_scope_t *parent,
    assert(parent != NULL);
    assert(data->root_scope != NULL);
 
-   cover_scope_t *s = xcalloc(sizeof(cover_scope_t));
+   cover_scope_t *s = pool_calloc(data->pool, sizeof(cover_scope_t));
 
    if (cover_is_branch(t)) {
       if (tree_kind(t) == T_ASSOC && tree_subkind(t) == A_OTHERS)
@@ -966,44 +1056,12 @@ cover_scope_t *cover_create_scope(cover_data_t *data, cover_scope_t *parent,
    else if (name == NULL)
       name = ident_sprintf("_S%u", parent->stmt_label++);
 
-   s->name       = name;
-   s->parent     = parent;
-   s->block_name = s->parent->block_name;
-   s->loc        = *tree_loc(t);
-   s->hier       = ident_prefix(s->parent->hier, name, '.');
-   s->emit       = cover_should_emit_scope(data, s, t);
-
-   if (s->sig_pos == 0)
-      s->sig_pos = parent->sig_pos;
-
-   APUSH(parent->children, s);
-   return s;
-}
-
-cover_scope_t *cover_create_instance(cover_data_t *data, cover_scope_t *parent,
-                                     tree_t block, tree_t unit)
-{
-
-   if (data == NULL)
-      return NULL;
-   else if (parent == NULL) {
-      assert(data->root_scope == NULL);
-
-      parent = data->root_scope = xcalloc(sizeof(cover_scope_t));
-      parent->name = parent->hier = lib_name(lib_work());
-   }
-
-   assert(is_concurrent_block(unit));
-   assert(tree_kind(block) == T_BLOCK);
-
-   cover_scope_t *s = xcalloc(sizeof(cover_scope_t));
-   s->name       = tree_ident(block);
-   s->parent     = parent;
-   s->loc        = *tree_loc(unit);
-   s->hier       = ident_prefix(s->parent->hier, s->name, '.');
-   s->emit       = cover_should_emit_scope(data, s, block);
-   s->block_name = ident_rfrom(tree_ident(unit), '.');
-   s->type       = CSCOPE_INSTANCE;
+   s->name   = name;
+   s->parent = parent;
+   s->block  = parent->block;
+   s->loc    = *tree_loc(t);
+   s->hier   = ident_prefix(parent->hier, name, '.');
+   s->emit   = cover_should_emit_scope(data, s);
 
    if (s->sig_pos == 0)
       s->sig_pos = parent->sig_pos;
@@ -1026,7 +1084,6 @@ static void cover_read_header(fbuf_t *f, cover_data_t *data)
 
    data->mask        = fbuf_get_uint(f);
    data->array_limit = fbuf_get_uint(f);
-   data->next_tag    = fbuf_get_uint(f);
 }
 
 static void cover_read_one_item(fbuf_t *f, loc_rd_ctx_t *loc_rd,
@@ -1063,19 +1120,18 @@ static void cover_read_one_item(fbuf_t *f, loc_rd_ctx_t *loc_rd,
       item->func_name = ident_read(ident_ctx);
 }
 
-static cover_scope_t *cover_read_scope(fbuf_t *f, ident_rd_ctx_t ident_ctx,
-                                       loc_rd_ctx_t *loc_ctx)
+static cover_scope_t *cover_read_scope(cover_data_t *db, fbuf_t *f,
+                                       ident_rd_ctx_t ident_ctx,
+                                       loc_rd_ctx_t *loc_ctx,
+                                       cover_block_t *b)
 {
-   cover_scope_t *s = xcalloc(sizeof(cover_scope_t));
-   s->type = fbuf_get_uint(f);
-   s->name = ident_read(ident_ctx);
-   s->hier = ident_read(ident_ctx);
+   cover_scope_t *s = pool_calloc(db->pool, sizeof(cover_scope_t));
+   s->name  = ident_read(ident_ctx);
+   s->hier  = ident_read(ident_ctx);
+   s->block = b;
 
    loc_read(&s->loc, loc_ctx);
 
-   if (s->type == CSCOPE_INSTANCE)
-      s->block_name = ident_read(ident_ctx);
-
    const int nitems = fbuf_get_uint(f);
    for (int i = 0; i < nitems; i++) {
       cover_item_t new;
@@ -1087,9 +1143,38 @@ static cover_scope_t *cover_read_scope(fbuf_t *f, ident_rd_ctx_t ident_ctx,
    for (;;) {
       const uint8_t ctrl = read_u8(f);
       switch (ctrl) {
+      case CTRL_PUSH_UNIT:
+         {
+            // FIXME: this check is required for merging
+            ident_t name = ident_read(ident_ctx);
+            cover_block_t *b = hash_get(db->blocks, name);
+            if (b == NULL) {
+               b = xcalloc(sizeof(cover_block_t));
+               b->name = name;
+               b->block_name = ident_read(ident_ctx);
+               b->kind = fbuf_get_uint(f);
+               b->next_tag = fbuf_get_uint(f);
+               b->self = cover_read_scope(db, f, ident_ctx, loc_ctx, b);
+
+               hash_put(db->blocks, b->name, b);
+
+               APUSH(s->children, b->self);
+            }
+            else {
+               (void)ident_read(ident_ctx);
+               (void)fbuf_get_uint(f);
+               (void)fbuf_get_uint(f);
+
+               cover_scope_t *child =
+                  cover_read_scope(db, f, ident_ctx, loc_ctx, b);
+               APUSH(s->children, child);
+            }
+         }
+         break;
       case CTRL_PUSH_SCOPE:
          {
-            cover_scope_t *child = cover_read_scope(f, ident_ctx, loc_ctx);
+            cover_scope_t *child =
+               cover_read_scope(db, f, ident_ctx, loc_ctx, b);
             APUSH(s->children, child);
          }
          break;
@@ -1106,6 +1191,8 @@ cover_data_t *cover_read_items(fbuf_t *f, uint32_t pre_mask)
    cover_data_t *data = xcalloc(sizeof(cover_data_t));
    cover_read_header(f, data);
    data->mask |= pre_mask;
+   data->blocks = hash_new(16);
+   data->pool = pool_new();
 
    loc_rd_ctx_t *loc_rd = loc_read_begin(f);
    ident_rd_ctx_t ident_ctx = ident_read_begin(f);
@@ -1115,7 +1202,7 @@ cover_data_t *cover_read_items(fbuf_t *f, uint32_t pre_mask)
       const uint8_t ctrl = read_u8(f);
       switch (ctrl) {
       case CTRL_PUSH_SCOPE:
-         data->root_scope = cover_read_scope(f, ident_ctx, loc_rd);
+         data->root_scope = cover_read_scope(data, f, ident_ctx, loc_rd, NULL);
          break;
       case CTRL_END_OF_FILE:
          eof = true;
@@ -1207,7 +1294,8 @@ void cover_merge_items(fbuf_t *f, cover_data_t *data, merge_mode_t mode)
       switch (ctrl) {
       case CTRL_PUSH_SCOPE:
          {
-            cover_scope_t *new = cover_read_scope(f, ident_ctx, loc_rd);
+            cover_scope_t *new =
+               cover_read_scope(data, f, ident_ctx, loc_rd, NULL);
             cover_merge_scope(data->root_scope, new, mode);
          }
          break;
@@ -1223,12 +1311,31 @@ void cover_merge_items(fbuf_t *f, cover_data_t *data, merge_mode_t mode)
    loc_read_end(loc_rd);
 }
 
-unsigned cover_count_items(cover_data_t *data)
+int32_t *cover_get_counters(cover_data_t *db, ident_t name)
 {
-   if (data == NULL)
-      return 0;
-   else
-      return data->next_tag;
+   if (db == NULL)
+      return NULL;
+
+   cover_block_t *b = hash_get(db->blocks, name);
+   if (b == NULL || b->next_tag == 0)
+      return NULL;
+
+   if (b->data == NULL)
+      b->data = pool_calloc(db->pool, b->next_tag *sizeof(int32_t));
+
+   return b->data;
+}
+
+cover_scope_t *cover_get_scope(cover_data_t *db, ident_t name)
+{
+   if (db == NULL)
+      return NULL;
+
+   cover_block_t *b = hash_get(db->blocks, name);
+   if (b == NULL)
+      return NULL;
+
+   return b->self;
 }
 
 void cover_bmask_to_bin_list(uint32_t bmask, text_buf_t *tb)
@@ -1278,3 +1385,45 @@ const char *cover_item_kind_str(cover_item_kind_t kind)
    assert(kind < ARRAY_LEN(item_kind_str));
    return item_kind_str[kind];
 }
+
+block_kind_t cover_block_kind(cover_scope_t *s)
+{
+   if (s->block == NULL)
+      return CBLOCK_NONE;  // Dummy root scope
+   else if (s->block->self == s)
+      return s->block->kind;
+   else
+      return CBLOCK_NONE;
+}
+
+bool cover_is_hier(cover_scope_t *s)
+{
+   switch (cover_block_kind(s)) {
+   case CBLOCK_INSTANCE:
+   case CBLOCK_PACKAGE:
+      assert(s->block->block_name != NULL);
+      return true;
+   default:
+      return false;
+   }
+}
+
+bool cover_bin_unreachable(cover_data_t *data, const cover_item_t *item)
+{
+   if ((data->mask & COVER_MASK_EXCLUDE_UNREACHABLE) == 0)
+      return false;
+
+   // Toggle items remember unreachability in run-time data. Must check
+   // item kind not to get false unreachability on statement items.
+   // Excludes both bins automatically!
+   if (item->kind == COV_ITEM_TOGGLE
+       && ((item->data & COV_FLAG_UNREACHABLE) != 0))
+      return true;
+
+   // Expression items remember unreachability as unreachable mask
+   if (item->kind == COV_ITEM_EXPRESSION
+       && ((item->flags & COV_FLAG_UNREACHABLE) != 0))
+      return true;
+
+   return false;
+}
