@@ -29,6 +29,7 @@
 #include "rt/rt.h"
 #include "rt/structs.h"
 #include "type.h"
+#include "vlog/vlog-number.h"
 
 #include <assert.h>
 #include <ctype.h>
@@ -485,6 +486,10 @@ void __nvc_do_exit(jit_exit_t which, jit_anchor_t *anchor, jit_scalar_t *args,
       __nvc_sched_waveform(anchor, args, tlab);
       break;
 
+   case JIT_EXIT_SCHED_INACTIVE:
+      x_sched_inactive();
+      break;
+
    case JIT_EXIT_SCHED_EVENT:
       {
          sig_shared_t *shared  = args[0].pointer;
@@ -495,6 +500,20 @@ void __nvc_do_exit(jit_exit_t which, jit_anchor_t *anchor, jit_scalar_t *args,
       }
       break;
 
+   case JIT_EXIT_ENABLE_TRIGGER:
+      {
+         rt_trigger_t *trigger = args[0].pointer;
+         x_enable_trigger(trigger);
+      }
+      break;
+
+   case JIT_EXIT_DISABLE_TRIGGER:
+      {
+         rt_trigger_t *trigger = args[0].pointer;
+         x_disable_trigger(trigger);
+      }
+      break;
+
    case JIT_EXIT_ALIAS_SIGNAL:
       {
          sig_shared_t *ss    = args[0].pointer;
@@ -599,6 +618,37 @@ void __nvc_do_exit(jit_exit_t which, jit_anchor_t *anchor, jit_scalar_t *args,
       }
       break;
 
+   case JIT_EXIT_SCHED_DEPOSIT:
+      {
+         sig_shared_t *shared = args[0].pointer;
+         int32_t       offset = args[1].integer;
+         int32_t       count  = args[2].integer;
+         jit_scalar_t  value  = { .integer = args[3].integer };
+         bool          scalar = args[4].integer;
+         int64_t       after  = args[5].integer;
+
+         if (scalar)
+            x_sched_deposit(shared, offset, count, &value.integer, after);
+         else
+            x_sched_deposit(shared, offset, count, value.pointer, after);
+      }
+      break;
+
+   case JIT_EXIT_PUT_DRIVER:
+      {
+         sig_shared_t *shared = args[0].pointer;
+         int32_t       offset = args[1].integer;
+         int32_t       count  = args[2].integer;
+         jit_scalar_t  value  = { .integer = args[3].integer };
+         bool          scalar = args[4].integer;
+
+         if (scalar)
+            x_put_driver(shared, offset, count, &value.integer);
+         else
+            x_put_driver(shared, offset, count, value.pointer);
+      }
+      break;
+
    case JIT_EXIT_PUT_CONVERSION:
       {
          rt_conv_func_t *cf     = args[0].pointer;
@@ -864,6 +914,16 @@ void __nvc_do_exit(jit_exit_t which, jit_anchor_t *anchor, jit_scalar_t *args,
       }
       break;
 
+   case JIT_EXIT_LEVEL_TRIGGER:
+      {
+         sig_shared_t *shared = args[0].pointer;
+         int32_t       offset = args[1].integer;
+         int32_t       count  = args[2].integer;
+
+         args[0].pointer = x_level_trigger(shared, offset, count);
+      }
+      break;
+
    case JIT_EXIT_ADD_TRIGGER:
       {
          void *trigger = args[0].pointer;
@@ -953,6 +1013,194 @@ void __nvc_do_exit(jit_exit_t which, jit_anchor_t *anchor, jit_scalar_t *args,
    thread->anchor = NULL;
 }
 
+DLLEXPORT
+void __nvc_vec4op(jit_vec_op_t op, jit_anchor_t *anchor, jit_scalar_t *args,
+                  int size)
+{
+   const int nwords = (size + 63) / 64;
+   assert(size > 64);
+
+   jit_thread_local_t *thread = jit_attach_thread(anchor);
+
+   switch (op) {
+   case JIT_VEC_CASE_EQ:
+   case JIT_VEC_CASE_NEQ:
+   case JIT_VEC_AND1:
+   case JIT_VEC_OR1:
+      {
+         const uint64_t *aleft  = args[0].pointer;
+         const uint64_t *bleft  = args[1].pointer;
+         const uint64_t *aright = args[2].pointer;
+         const uint64_t *bright = args[3].pointer;
+
+         int aresult, bresult;
+         switch (op) {
+         case JIT_VEC_CASE_EQ:
+            aresult = memcmp(aleft, aright, nwords * sizeof(uint64_t)) == 0
+               && memcmp(bleft, bright, nwords * sizeof(uint64_t)) == 0;
+            bresult = 0;
+            break;
+         case JIT_VEC_CASE_NEQ:
+            aresult = memcmp(aleft, aright, nwords * sizeof(uint64_t)) != 0
+               || memcmp(bleft, bright, nwords * sizeof(uint64_t)) != 0;
+            bresult = 0;
+            break;
+         case JIT_VEC_AND1:
+            bresult = vec2_or1(size, bleft);
+            aresult = vec2_and1(size, aleft) | bresult;
+            break;
+         case JIT_VEC_OR1:
+            bresult = vec2_or1(size, bleft);
+            aresult = vec2_or1(size, aleft) | bresult;
+            break;
+         default:
+            should_not_reach_here();
+         }
+
+         args[0].integer = aresult;
+         args[1].integer = bresult;
+      }
+      break;
+   case JIT_VEC_ZEXT:
+      {
+         void *ptr = jit_mspace_alloc(nwords * 2 * sizeof(uint64_t));
+         uint64_t *aresult = ptr, *bresult = aresult + nwords;
+
+         if (op == JIT_VEC_ZEXT) {
+            const int arg_size = args[2].integer;
+            if (arg_size <= 64) {
+               aresult[0] = args[0].integer;
+               bresult[0] = args[1].integer;
+
+               for (int i = 1; i < nwords; i++)
+                  aresult[i] = bresult[i] = 0;
+            }
+            else {
+               const uint64_t *asrc = args[0].pointer;
+               const uint64_t *bsrc = args[1].pointer;
+
+               for (int i = 0; i < nwords; i++) {
+                  if (i < (arg_size + 63) / 64) {
+                     aresult[i] = asrc[i];
+                     bresult[i] = bsrc[i];
+                  }
+                  else
+                     aresult[i] = bresult[i] = 0;
+               }
+            }
+         }
+
+         args[0].pointer = aresult;
+         args[1].pointer = bresult;
+      }
+      break;
+   case JIT_VEC_INSERT:
+      {
+         void *ptr = jit_mspace_alloc(nwords * 2 * sizeof(uint64_t));
+         uint64_t *aresult = ptr, *bresult = aresult + nwords;
+
+         const uint64_t *afull = args[0].pointer;
+         const uint64_t *bfull = args[1].pointer;
+
+         memcpy(aresult, afull, nwords * sizeof(uint64_t));
+         memcpy(bresult, bfull, nwords * sizeof(uint64_t));
+
+         const int part_size = args[4].integer;
+         const int pos       = args[5].integer;
+
+         const int bitpos = size - part_size - pos;
+
+         for (int i = 0; i < part_size; i += 64) {
+            uint64_t apart, bpart;
+            if (part_size <= 64) {
+               apart = args[2].integer;
+               bpart = args[3].integer;
+            }
+            else {
+               apart = ((const uint64_t *)args[2].pointer)[i / 64];
+               bpart = ((const uint64_t *)args[3].pointer)[i / 64];
+            }
+
+            const int opos = bitpos + i;
+
+            uint64_t mask = ~UINT64_C(0);
+            if (part_size - i < 64)
+               mask >>= 64 - (part_size - i);
+            mask <<= opos % 64;
+
+            aresult[opos / 64] &= ~mask;
+            bresult[opos / 64] &= ~mask;
+
+            aresult[opos / 64] |= apart << (opos % 64);
+            bresult[opos / 64] |= bpart << (opos % 64);
+
+            const int msb = opos + MIN(64, part_size - i) - 1;
+
+            if (msb / 64 != opos / 64) {
+               const int shift = 64 - opos;
+               apart >>= shift;
+               bpart >>= shift;
+
+               aresult[msb / 64] &= ~UINT64_C(0) >> (64 - shift);
+               bresult[msb / 64] &= ~UINT64_C(0) >> (64 - shift);
+
+               aresult[msb / 64] |= apart;
+               bresult[msb / 64] |= bpart;
+            }
+         }
+
+         args[0].pointer = aresult;
+         args[1].pointer = bresult;
+      }
+      break;
+   default:
+      {
+         void *ptr = jit_mspace_alloc(nwords * 2 * sizeof(uint64_t));
+         uint64_t *aresult = ptr, *bresult = aresult + nwords;
+
+         const uint64_t *a1 = args[0].pointer;
+         const uint64_t *b1 = args[1].pointer;
+         const uint64_t *a2 = args[2].pointer;
+         const uint64_t *b2 = args[3].pointer;
+
+         memcpy(aresult, a1, nwords * sizeof(uint64_t));
+         memcpy(bresult, b1, nwords * sizeof(uint64_t));
+
+         switch (op) {
+         case JIT_VEC_ADD:
+            vec4_add(size, aresult, bresult, a2, b2);
+            break;
+         case JIT_VEC_MUL:
+            vec4_mul(size, aresult, bresult, a2, b2);
+            break;
+         case JIT_VEC_DIV:
+            vec4_div(size, aresult, bresult, a2, b2);
+            break;
+         case JIT_VEC_MOD:
+            vec4_mod(size, aresult, bresult, a2, b2);
+            break;
+         case JIT_VEC_SHL:
+            vec4_shl(size, aresult, bresult, a2, b2);
+            break;
+         case JIT_VEC_SHR:
+            vec4_shr(size, aresult, bresult, a2, b2);
+            break;
+         case JIT_VEC_NOT:
+            vec4_inv(size, aresult, bresult);
+            break;
+         default:
+            should_not_reach_here();
+         }
+
+         args[0].pointer = aresult;
+         args[1].pointer = bresult;
+      }
+      break;
+   }
+
+   thread->anchor = NULL;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // Entry points from AOT compiled code
 
@@ -1016,26 +1264,58 @@ object_t *__nvc_get_object(const char *unit, ptrdiff_t offset)
 DLLEXPORT
 void __nvc_pack(const uint8_t *src, int32_t size, jit_scalar_t *args)
 {
-   assert(size <= 64);  // TODO
+   if (size > 64) {
+      const int nwords = (size + 63) / 64;
+      uint64_t *mem = jit_mspace_alloc(nwords * 2 * sizeof(uint64_t));
+      uint64_t *abits = mem, *bbits = mem + nwords;
 
-   uint64_t abits = 0, bbits = 0;
+      for (int i = 0; i < nwords * 2; i++)
+         mem[i] = 0;
 
-   for (size_t i = 0; i < size; i++) {
-      abits = (abits << 1) | (src[i] & 1);
-      bbits = (bbits << 1) | ((src[i] >> 1) & 1);
+      for (int i = 0; i < size; i++) {
+         const int pos = size - 1 - i;
+         abits[pos / 64] |= (uint64_t)(src[i] & 1) << (pos % 64);
+         bbits[pos / 64] |= (uint64_t)((src[i] >> 1) & 1) << (pos % 64);
+      }
+
+      args[0].pointer = abits;
+      args[1].pointer = bbits;
    }
+   else {
+      uint64_t abits = 0, bbits = 0;
 
-   args[0].integer = abits;
-   args[1].integer = bbits;
+      for (int i = 0; i < size; i++) {
+         abits = (abits << 1) | (src[i] & 1);
+         bbits = (bbits << 1) | ((src[i] >> 1) & 1);
+      }
+
+      args[0].integer = abits;
+      args[1].integer = bbits;
+   }
 }
 
 DLLEXPORT
-void __nvc_unpack(uint64_t abits, uint64_t bbits, jit_scalar_t *args)
+void __nvc_unpack(jit_scalar_t aval, jit_scalar_t bval, jit_scalar_t *args)
 {
    uint8_t *dest = args[0].pointer;
    size_t size = args[1].integer;
    uint8_t strength = args[2].integer;
 
-   for (size_t i = 0; i < size; i++, abits >>= 1, bbits >>= 1)
-      dest[size - i - 1] = (abits & 1) | ((bbits & 1) << 1) | strength;
+   if (size > 64) {
+      const uint64_t *abits = aval.pointer;
+      const uint64_t *bbits = bval.pointer;
+
+      for (int i = 0; i < size; i++) {
+         const uint64_t abit = (abits[i / 64] >> (i % 64)) & 1;
+         const uint64_t bbit = (bbits[i / 64] >> (i % 64)) & 1;
+         dest[size - i - 1] = abit | (bbit << 1) | strength;
+      }
+   }
+   else {
+      uint64_t abits = aval.integer;
+      uint64_t bbits = bval.integer;
+
+      for (int i = 0; i < size; i++, abits >>= 1, bbits >>= 1)
+         dest[size - i - 1] = (abits & 1) | ((bbits & 1) << 1) | strength;
+   }
 }
