@@ -155,6 +155,7 @@ const char *mir_op_string(mir_op_t op)
       [MIR_OP_FUNCTION_TRIGGER] = "function trigger",
       [MIR_OP_OR_TRIGGER] = "or trigger",
       [MIR_OP_CMP_TRIGGER] = "cmp trigger",
+      [MIR_OP_LEVEL_TRIGGER] = "level trigger",
       [MIR_OP_INSTANCE_NAME] = "instance name",
       [MIR_OP_LAST_EVENT] = "last event",
       [MIR_OP_LAST_ACTIVE] = "last active",
@@ -170,6 +171,13 @@ const char *mir_op_string(mir_op_t op)
       [MIR_OP_BINARY] = "vector binary",
       [MIR_OP_UNARY] = "vector unary",
       [MIR_OP_DIR_CHECK] = "dir check",
+      [MIR_OP_INSERT] = "insert",
+      [MIR_OP_TEST] = "test",
+      [MIR_OP_EXTRACT] = "extract",
+      [MIR_OP_SCHED_PROCESS] = "sched process",
+      [MIR_OP_SCHED_INACTIVE] = "sched inactive",
+      [MIR_OP_SCHED_DEPOSIT] = "sched deposit",
+      [MIR_OP_PUT_DRIVER] = "put driver",
    };
 
    return map[op];
@@ -506,11 +514,16 @@ static int mir_dump_vector_op(mir_unit_t *mu, mir_value_t value)
    case MIR_VEC_LOG_NEQ:  return printf(" != ");
    case MIR_VEC_CASE_EQ:  return printf(" === ");
    case MIR_VEC_CASE_NEQ: return printf(" !== ");
+   case MIR_VEC_CASEX_EQ: return printf(" casex == ");
    case MIR_VEC_ADD:      return printf(" + ");
    case MIR_VEC_SUB:      return printf(" - ");
    case MIR_VEC_MUL:      return printf(" * ");
+   case MIR_VEC_DIV:      return printf(" / ");
+   case MIR_VEC_MOD:      return printf(" %% ");
    case MIR_VEC_SLL:      return printf(" << ");
    case MIR_VEC_SRL:      return printf(" >> ");
+   case MIR_VEC_SLA:      return printf(" <<< ") ;
+   case MIR_VEC_SRA:      return printf(" >>> ");
    default: return printf(" <%d> ", value.id);
    }
 }
@@ -768,16 +781,6 @@ void mir_annotate(mir_unit_t *mu, const mir_annotate_t *cb, void *ctx)
             }
             break;
 
-         case MIR_OP_RETURN:
-            {
-               printf("%s", mir_op_string(n->op));
-               if (n->nargs > 0) {
-                  printf(" ");
-                  mir_dump_value(mu, n->args[0], cb, ctx);
-               }
-            }
-            break;
-
          case MIR_OP_CONST:
             {
                col += mir_dump_value(mu, result, cb, ctx);
@@ -891,6 +894,13 @@ void mir_annotate(mir_unit_t *mu, const mir_annotate_t *cb, void *ctx)
             }
             break;
 
+         case MIR_OP_SCHED_PROCESS:
+            {
+               printf("%s after ", mir_op_string(n->op));
+               mir_dump_value(mu, n->args[0], cb, ctx);
+            }
+            break;
+
          case MIR_OP_SELECT:
             {
                col += mir_dump_value(mu, result, cb, ctx);
@@ -1369,8 +1379,10 @@ void mir_annotate(mir_unit_t *mu, const mir_annotate_t *cb, void *ctx)
             {
                printf("%s on ", mir_op_string(n->op));
                mir_dump_arg(mu, result, 0, cb, ctx);
-               printf(" count " );
-               mir_dump_arg(mu, result, 1, cb, ctx);
+               if (n->nargs > 1) {
+                  printf(" count " );
+                  mir_dump_arg(mu, result, 1, cb, ctx);
+               }
             }
             break;
 
@@ -1554,6 +1566,7 @@ void mir_annotate(mir_unit_t *mu, const mir_annotate_t *cb, void *ctx)
 
          case MIR_OP_FORCE:
          case MIR_OP_DEPOSIT_SIGNAL:
+         case MIR_OP_PUT_DRIVER:
             {
                printf("%s ", mir_op_string(n->op));
                mir_dump_arg(mu, result, 0, cb, ctx);
@@ -1564,8 +1577,23 @@ void mir_annotate(mir_unit_t *mu, const mir_annotate_t *cb, void *ctx)
             }
             break;
 
+         case MIR_OP_SCHED_DEPOSIT:
+            {
+               printf("%s ", mir_op_string(n->op));
+               mir_dump_arg(mu, result, 0, cb, ctx);
+               printf(" count ");
+               mir_dump_arg(mu, result, 1, cb, ctx);
+               printf(" values ");
+               mir_dump_arg(mu, result, 2, cb, ctx);
+               printf(" after ");
+               mir_dump_arg(mu, result, 3, cb, ctx);
+            }
+            break;
+
          case MIR_OP_UNREACHABLE:
          case MIR_OP_POP_SCOPE:
+         case MIR_OP_SCHED_INACTIVE:
+         case MIR_OP_RETURN:
             {
                printf("%s", mir_op_string(n->op));
                if (n->nargs > 0) {
@@ -1670,6 +1698,18 @@ void mir_annotate(mir_unit_t *mu, const mir_annotate_t *cb, void *ctx)
             }
             break;
 
+         case MIR_OP_LEVEL_TRIGGER:
+            {
+               col += mir_dump_value(mu, result, cb, ctx);
+               col += printf(" := %s ", mir_op_string(n->op));
+               col += mir_dump_value(mu, n->args[0], cb, ctx);
+               col += printf(" count ");
+               col += mir_dump_value(mu, n->args[1], cb, ctx);
+               mir_dump_type(mu, col, n->type);
+               mir_dump_stamp(mu, n->type, n->stamp);
+            }
+            break;
+
          case MIR_OP_ENTER_STATE:
             {
                printf("%s ", mir_op_string(n->op));
@@ -1735,6 +1775,7 @@ void mir_annotate(mir_unit_t *mu, const mir_annotate_t *cb, void *ctx)
             break;
 
          case MIR_OP_PACK:
+         case MIR_OP_TEST:
             {
                col += mir_dump_value(mu, result, cb, ctx);
                col += printf(" := %s ", mir_op_string(n->op));
@@ -1767,6 +1808,32 @@ void mir_annotate(mir_unit_t *mu, const mir_annotate_t *cb, void *ctx)
             }
             break;
 
+         case MIR_OP_INSERT:
+            {
+               col += mir_dump_value(mu, result, cb, ctx);
+               col += printf(" := %s ", mir_op_string(n->op));
+               col += mir_dump_arg(mu, result, 0, cb, ctx);
+               col += printf(" into ");
+               col += mir_dump_arg(mu, result, 1, cb, ctx);
+               col += printf(" at ");
+               col += mir_dump_arg(mu, result, 2, cb, ctx);
+               mir_dump_type(mu, col, n->type);
+               mir_dump_stamp(mu, n->type, n->stamp);
+            }
+            break;
+
+         case MIR_OP_EXTRACT:
+            {
+               col += mir_dump_value(mu, result, cb, ctx);
+               col += printf(" := %s from ", mir_op_string(n->op));
+               col += mir_dump_arg(mu, result, 0, cb, ctx);
+               col += printf(" at ");
+               col += mir_dump_arg(mu, result, 1, cb, ctx);
+               mir_dump_type(mu, col, n->type);
+               mir_dump_stamp(mu, n->type, n->stamp);
+            }
+            break;
+
 #ifdef DEBUG
          case MIR_OP_COMMENT:
             {
