@@ -19,6 +19,7 @@
 #include "common.h"
 #include "cov/cov-api.h"
 #include "diag.h"
+#include "ident.h"
 #include "jit/jit-llvm.h"
 #include "jit/jit.h"
 #include "lib.h"
@@ -90,7 +91,7 @@ const char version_string[] =
 
 static int process_command(int argc, char **argv, cmd_state_t *state);
 static int parse_int(const char *str);
-static jit_t *get_jit(unit_registry_t *ur, mir_context_t *mc);
+static jit_t *get_jit(cmd_state_t *state);
 
 static ident_t to_unit_name(const char *str)
 {
@@ -161,6 +162,7 @@ static void do_file_list(const char *file, jit_t *jit, unit_registry_t *ur,
    else if ((f = fopen(file, "r")) == NULL)
       fatal_errno("failed to open %s", file);
 
+   LOCAL_TEXT_BUF tb = tb_new();
    char *line = NULL;
    size_t nchars, bufsz = 0;
    while ((nchars = getline(&line, &bufsz, f)) != -1) {
@@ -174,7 +176,30 @@ static void do_file_list(const char *file, jit_t *jit, unit_registry_t *ur,
       if (strlen(line) == 0)
          continue;
 
-      analyse_file(line, jit, ur, mc);
+      tb_rewind(tb);
+
+      for (char *p = line; *p;) {
+         if (*p == '$') {  // Interpolate environment variable
+            const char *start = ++p;
+            while (isalnum_iso88591(*p) || *p == '_') p++;
+            const char tail = *p;
+            *p++ = '\0';
+
+            const char *envvar = getenv(start);
+            if (envvar == NULL) {
+               warnf("environment variable $%s not set", start);
+               tb_cat(tb, start - 1);
+            }
+            else
+               tb_cat(tb, envvar);
+
+            tb_append(tb, tail);
+         }
+         else
+            tb_append(tb, *p++);
+      }
+
+      analyse_file(tb_get(tb), jit, ur, mc);
    }
 
    free(line);
@@ -196,13 +221,14 @@ static int analyse(int argc, char **argv, cmd_state_t *state)
       { "no-save",         no_argument,       0, 'N' },
       { "single-unit",     no_argument,       0, 'u' },
       { "preserve-case",   no_argument,       0, 'p' },
+      { "keywords",        required_argument, 0, 'k' },
       { 0, 0, 0, 0 }
    };
 
    const int next_cmd = scan_cmd(2, argc, argv);
    int c, index = 0, error_limit = 20;
    const char *file_list = NULL;
-   const char *spec = ":D:f:";
+   const char *spec = ":D:f:I:";
    bool no_save = false;
 
    while ((c = getopt_long(next_cmd, argv, spec, long_options, &index)) != -1) {
@@ -246,12 +272,24 @@ static int analyse(int argc, char **argv, cmd_state_t *state)
       case 'N':
          no_save = true;
          break;
+      case 'I':
+         add_include_dir(optarg);
+         break;
       case 'u':
          opt_set_int(OPT_SINGLE_UNIT, 1);
          break;
       case 'p':
          opt_set_int(OPT_PRESERVE_CASE, 1);
          break;
+      case 'k':
+         {
+            vlog_version_t vers;
+            if (parse_verilog_version(optarg, &vers))
+               push_keywords(vers);
+            else
+               fatal("'%s' is not a valid Verilog version", optarg);
+         }
+         break;
       default:
          should_not_reach_here();
       }
@@ -265,7 +303,7 @@ static int analyse(int argc, char **argv, cmd_state_t *state)
    if (state->registry == NULL)
       state->registry = unit_registry_new(state->mir);
 
-   jit_t *jit = jit_new(state->registry, state->mir);
+   jit_t *jit = jit_new(state->registry, state->mir, NULL);
 
    if (file_list != NULL)
       do_file_list(file_list, jit, state->registry, state->mir);
@@ -416,7 +454,7 @@ static int elaborate(int argc, char **argv, cmd_state_t *state)
       { "verbose",         no_argument,       0, 'V' },
       { "no-save",         no_argument,       0, 'N' },
       { "jit",             no_argument,       0, 'j' },
-      { "precompile",      no_argument,       0, 'p' },
+      { "precompile",      no_argument,       0, 'p' },   // DEPRECATED 1.18
       { "no-collapse",     no_argument,       0, 'C' },
       { "trace",           no_argument,       0, 't' },
       { 0, 0, 0, 0 }
@@ -463,6 +501,8 @@ static int elaborate(int argc, char **argv, cmd_state_t *state)
          use_jit = true;
          break;
       case 'p':
+         warnf("the $bold$--precompile$$ option is deprecated and will be "
+               "removed in a future release");
          use_jit = false;
          break;
       case 'g':
@@ -504,12 +544,11 @@ static int elaborate(int argc, char **argv, cmd_state_t *state)
    progress("loading top-level unit");
 
    char *cover_default LOCAL = NULL;
-   cover_data_t *cover = NULL;
    if (cover_mask != 0) {
-      cover = cover_data_init(cover_mask, cover_array_limit, threshold);
+      state->cover = cover_data_init(cover_mask, cover_array_limit, threshold);
 
       if (cover_spec_file != NULL)
-         cover_load_spec_file(cover, cover_spec_file);
+         cover_load_spec_file(state->cover, cover_spec_file);
 
       if (meta.cover_file == NULL) {
          cover_default = xasprintf("%s.ncdb", state->top_level_arg);
@@ -544,14 +583,14 @@ static int elaborate(int argc, char **argv, cmd_state_t *state)
 
    state->mir = mir_context_new();
    state->registry = unit_registry_new(state->mir);
-   state->jit = get_jit(state->registry, state->mir);
-   state->model = model_new(state->jit, cover);
+   state->jit = get_jit(state);
+   state->model = model_new(state->jit, state->cover);
 
    if (state->vhpi == NULL)
       state->vhpi = vhpi_context_new();
 
-   tree_t top = elab(obj, state->jit, state->registry, state->mir, cover,
-                     NULL, state->model);
+   tree_t top = elab(obj, state->jit, state->registry, state->mir,
+                     state->cover, NULL, state->model);
 
    if (top == NULL)
       return EXIT_FAILURE;
@@ -560,10 +599,13 @@ static int elaborate(int argc, char **argv, cmd_state_t *state)
 
    progress("elaborating design");
 
-   if (cover != NULL) {
-      fbuf_t *covdb = fbuf_open(meta.cover_file, FBUF_OUT, FBUF_CS_NONE);
-      cover_dump_items(cover, covdb, COV_DUMP_ELAB, NULL);
-      fbuf_close(covdb, NULL);
+   if (state->cover != NULL) {
+      fbuf_t *f = fbuf_open(meta.cover_file, FBUF_OUT, FBUF_CS_NONE);
+      if (f == NULL)
+         fatal_errno("failed to open coverage database: %s", meta.cover_file);
+
+      cover_dump_items(state->cover, f, COV_DUMP_ELAB);
+      fbuf_close(f, NULL);
 
       progress("dumping coverage data");
    }
@@ -590,7 +632,7 @@ static int elaborate(int argc, char **argv, cmd_state_t *state)
    else if (!no_save)
       cgen(top, state->registry, state->mir, state->jit, CGEN_JIT_PACK);
 
-   if (!use_jit || cover != NULL) {
+   if (!use_jit || state->cover != NULL) {
       // Must discard current JIT state to load AOT library later
       model_free(state->model);
       jit_free(state->jit);
@@ -687,9 +729,9 @@ static void ctrl_c_handler(void *arg)
    model_interrupt(model);
 }
 
-static jit_t *get_jit(unit_registry_t *ur, mir_context_t *mc)
+static jit_t *get_jit(cmd_state_t *state)
 {
-   jit_t *jit = jit_new(ur, mc);
+   jit_t *jit = jit_new(state->registry, state->mir, state->cover);
 
 #if defined HAVE_LLVM && 1
    jit_register_llvm_plugin(jit);
@@ -719,7 +761,7 @@ static int plusarg_cmp(const void *lptr, const void *rptr)
       return lptr - rptr;
 }
 
-static cover_data_t *load_coverage(const unit_meta_t *meta, jit_t *j)
+static cover_data_t *load_coverage(const unit_meta_t *meta)
 {
    if (meta->cover_file == NULL)
       return NULL;
@@ -730,10 +772,6 @@ static cover_data_t *load_coverage(const unit_meta_t *meta, jit_t *j)
 
    cover_data_t *db = cover_read_items(f, 0);
 
-   // Pre-allocate coverage counters
-   const int n_tags = cover_count_items(db);
-   jit_get_cover_mem(j, n_tags);
-
    fbuf_close(f, NULL);
    return db;
 }
@@ -746,9 +784,7 @@ static void emit_coverage(const unit_meta_t *meta, jit_t *j, cover_data_t *db)
    if (f == NULL)
       fatal_errno("failed to open coverage database: %s", meta->cover_file);
 
-   const int n_tags = cover_count_items(db);
-   const int32_t *counts = jit_sync_cover_mem(j, n_tags);
-   cover_dump_items(db, f, COV_DUMP_RUNTIME, counts);
+   cover_dump_items(db, f, COV_DUMP_RUNTIME);
 
    fbuf_close(f, NULL);
 }
@@ -953,6 +989,9 @@ static int run_cmd(int argc, char **argv, cmd_state_t *state)
    if (opt_get_size(OPT_HEAP_SIZE) < 0x100000)
       warnf("recommended heap size is at least 1M");
 
+   if (state->cover == NULL)
+      state->cover = load_coverage(meta);
+
    if (state->mir == NULL)
       state->mir = mir_context_new();
 
@@ -960,7 +999,7 @@ static int run_cmd(int argc, char **argv, cmd_state_t *state)
       state->registry = unit_registry_new(state->mir);
 
    if (state->jit == NULL)
-      state->jit = get_jit(state->registry, state->mir);
+      state->jit = get_jit(state);
 
 #ifdef ENABLE_LLVM
    jit_load_dll(state->jit, tree_ident(top));
@@ -968,9 +1007,6 @@ static int run_cmd(int argc, char **argv, cmd_state_t *state)
 
    load_jit_pack(state->jit, top);
 
-   if (state->cover == NULL)
-      state->cover = load_coverage(meta, state->jit);
-
    if (state->model == NULL) {
       state->model = model_new(state->jit, state->cover);
       create_scope(state->model, top, NULL);
@@ -1002,8 +1038,8 @@ static int run_cmd(int argc, char **argv, cmd_state_t *state)
       wave_dumper_restart(dumper, state->model, state->jit);
 
    if (opt_get_int(OPT_IEEE_WARNINGS) == IEEE_WARNINGS_OFF_AT_0)
-      model_set_global_cb(state->model, RT_END_TIME_STEP,
-                          enable_ieee_warnings_cb, state);
+      model_set_phase_cb(state->model, END_TIME_STEP,
+                         enable_ieee_warnings_cb, state);
 
    model_run(state->model, stop_time);
 
@@ -1143,6 +1179,7 @@ static void list_walk_fn(lib_t lib, ident_t ident, int kind, void *context)
    case T_PACK_INST: pretty = "Instantiated package"; break;
    case T_CONFIGURATION: pretty = "Configuration"; break;
    case T_CONTEXT: pretty = "Context"; break;
+   case T_LAST_TREE_KIND: pretty = "Verilog"; break;
    }
 
    printf("%-30s  : %s\n", istr(ident), pretty);
@@ -1351,11 +1388,17 @@ static void dump_one_unit(ident_t name, bool add_elab, bool add_body)
    else if (add_body)
       name = ident_prefix(name, well_known(W_BODY), '-');
 
-   tree_t top = lib_get(lib_work(), name);
-   if (top == NULL)
+   object_t *obj = lib_get_generic(lib_work(), name, NULL);
+   if (obj == NULL)
       fatal("%s not analysed", istr(name));
 
-   dump(top);
+   tree_t t = tree_from_object(obj);
+   if (t != NULL)
+      dump(t);
+
+   vlog_node_t v = vlog_from_object(obj);
+   if (v != NULL)
+      vlog_dump(v, 0);
 }
 
 static int dump_cmd(int argc, char **argv, cmd_state_t *state)
@@ -1432,7 +1475,7 @@ static int do_cmd(int argc, char **argv, cmd_state_t *state)
       state->registry = unit_registry_new(state->mir);
 
    if (state->jit == NULL)
-      state->jit = get_jit(state->registry, state->mir);
+      state->jit = get_jit(state);
 
 #ifdef ENABLE_TCL
    tcl_shell_t *sh = shell_new(state->jit);
@@ -1507,7 +1550,7 @@ static int interact_cmd(int argc, char **argv, cmd_state_t *state)
       state->registry = unit_registry_new(state->mir);
 
    if (state->jit == NULL)
-      state->jit = get_jit(state->registry, state->mir);
+      state->jit = get_jit(state);
 
 #ifdef ENABLE_TCL
    tcl_shell_t *sh = shell_new(state->jit);
@@ -1674,7 +1717,7 @@ static int coverage_cmd(int argc, char **argv, cmd_state_t *state)
    if (out_db) {
       progress("Saving merged coverage database to: %s", out_db);
       fbuf_t *f = fbuf_open(out_db, FBUF_OUT, FBUF_CS_NONE);
-      cover_dump_items(cover, f, COV_DUMP_PROCESSING, NULL);
+      cover_dump_items(cover, f, COV_DUMP_PROCESSING);
       fbuf_close(f, NULL);
    }
 
@@ -1691,10 +1734,7 @@ static int coverage_cmd(int argc, char **argv, cmd_state_t *state)
    if (export_file && cover) {
       progress("Exporting XML coverage report");
 
-      FILE *f = fopen(export_file, "w");
-      if (f == NULL)
-         fatal_errno("cannot open %s", export_file);
-
+      FILE *f = create_file("%s", export_file);
       cover_export_cobertura(cover, f, NULL);
       fclose(f);
    }
@@ -1760,7 +1800,7 @@ static int gui_cmd(int argc, char **argv, cmd_state_t *state)
       state->mir = mir_context_new();
 
    if (state->jit == NULL)
-      state->jit = get_jit(state->registry, state->mir);
+      state->jit = get_jit(state);
 
    start_server(kind, state->jit, top, NULL, NULL, init_cmd);
 
@@ -1902,8 +1942,8 @@ static int cover_export_cmd(int argc, char **argv, cmd_state_t *state)
    }
 
    FILE *file = stdout;
-   if (output != NULL && (file = fopen(output, "w")) == NULL)
-      fatal_errno("cannot create %s", output);
+   if (output != NULL)
+      file = create_file("%s", output);
 
    switch (format) {
    case COBERTURA:
@@ -2047,7 +2087,7 @@ static int cover_merge_cmd(int argc, char **argv, cmd_state_t *state)
    progress("saving merged coverage database to %s", out_db);
 
    fbuf_t *f = fbuf_open(out_db, FBUF_OUT, FBUF_CS_NONE);
-   cover_dump_items(cover, f, COV_DUMP_PROCESSING, NULL);
+   cover_dump_items(cover, f, COV_DUMP_PROCESSING);
    fbuf_close(f, NULL);
 
    argc -= next_cmd - 1;
@@ -2067,7 +2107,7 @@ static int preprocess_cmd(int argc, char **argv, cmd_state_t *state)
    const int next_cmd = scan_cmd(2, argc, argv);
 
    int c, index;
-   const char *spec = ":D:";
+   const char *spec = ":D:I:";
    while ((c = getopt_long(argc, argv, spec, long_options, &index)) != -1) {
       switch (c) {
       case 'u':
@@ -2076,6 +2116,9 @@ static int preprocess_cmd(int argc, char **argv, cmd_state_t *state)
       case 'D':
          parse_pp_define(optarg);
          break;
+      case 'I':
+         add_include_dir(optarg);
+         break;
       case ':':
          missing_argument("preprocess", argv);
       case '?':
@@ -2179,6 +2222,9 @@ static void usage(void)
              "Set preprocessor symbol NAME to VALUE" },
            { "--error-limit=NUM", "Stop after NUM errors" },
            { "-f, --files=LIST", "Read files to analyse from LIST" },
+           { "-I DIR", "Add DIR to list of Verilog include directories" },
+           { "--keywords=VERSION",
+             "Use keywords from specified Verilog version" },
            { "--no-save", "Do not save analysed design units" },
            { "--preserve-case",
              "Preserve the original case of VHDL identifiers" },
@@ -2618,5 +2664,8 @@ int main(int argc, char **argv)
    if (state.mir != NULL)
       mir_context_free(state.mir);
 
+   if (state.cover != NULL)
+      cover_data_free(state.cover);
+
    return ret;
 }
