@@ -91,6 +91,8 @@ typedef struct _vcode_unit *vcode_unit_t;
 typedef struct _cover_data  cover_data_t;
 typedef struct _cover_item  cover_item_t;
 typedef struct _cover_scope cover_scope_t;
+typedef struct _cover_block cover_block_t;
+typedef struct _cover_rpt   cover_rpt_t;
 
 typedef struct loc_wr_ctx loc_wr_ctx_t;
 typedef struct loc_rd_ctx loc_rd_ctx_t;
