@@ -1920,8 +1920,13 @@ void unmap_file(void *ptr, size_t size)
 #endif
 }
 
-void make_dir(const char *path)
+void make_dir(const char *fmt, ...)
 {
+   va_list ap;
+   va_start(ap, fmt);
+   char *path LOCAL = xvasprintf(fmt, ap);
+   va_end(ap);
+
 #ifdef __MINGW32__
    if (!CreateDirectory(path, NULL) && (GetLastError() != ERROR_ALREADY_EXISTS))
       fatal_errno("mkdir: %s", path);
@@ -1931,6 +1936,20 @@ void make_dir(const char *path)
 #endif
 }
 
+FILE *create_file(const char *fmt, ...)
+{
+   va_list ap;
+   va_start(ap, fmt);
+   char *path LOCAL = xvasprintf(fmt, ap);
+   va_end(ap);
+
+   FILE *f = fopen(path, "w");
+   if (f == NULL)
+      fatal_errno("cannot create %s", path);
+
+   return f;
+}
+
 uint64_t get_timestamp_ns(void)
 {
 #if defined __MINGW32__
@@ -2072,26 +2091,6 @@ void __array_resize_slow(void **ptr, uint32_t *limit, uint32_t count,
    }
 }
 
-char *search_path(const char *name)
-{
-   const char *path = getenv("PATH");
-   if (path == NULL)
-      return xstrdup(name);
-
-   char LOCAL *tmp = xstrdup(path);
-   for (char *p = strtok(tmp, ":"); p; p = strtok(NULL, ":")) {
-      char *full = xasprintf("%s"DIR_SEP"%s", p, name);
-
-      struct stat sb;
-      if (stat(full, &sb) == 0)
-         return full;
-
-      free(full);
-   }
-
-   return xstrdup(name);
-}
-
 bool get_exe_path(text_buf_t *tb)
 {
 #if defined __linux__
