@@ -21,7 +21,9 @@
 #include "tree.h"
 #include "type.h"
 #include "vhdl/vhdl-lower.h"
-#include "vhdl/vhdl-priv.h"
+#include "vhdl/vhdl-util.h"
+
+#include <assert.h>
 
 static void fill_array_type_info(mir_unit_t *mu, type_t type, type_info_t *ti)
 {
@@ -30,12 +32,17 @@ static void fill_array_type_info(mir_unit_t *mu, type_t type, type_info_t *ti)
    for (type_t e = type_elem(type);
         type_is_array(e) && !type_const_bounds(e);
         e = type_elem(e))
-      ti->udims += dimension_of(e);
+      ti->udims += dimension_of(e);  // XXX: equal to elem->udims?
 
-   const type_info_t *elem = type_info(mu, type_elem_recur(type));
+   const type_info_t *elem = type_info(mu, type_elem(type));
+   const type_info_t *base = type_info(mu, type_elem_recur(type));
+
+   if (elem->size != SIZE_MAX && elem->stride != SIZE_MAX)
+      ti->stride = elem->size * elem->stride;
+   else
+      ti->stride = SIZE_MAX;
 
    if (type_const_bounds(type)) {
-      ti->size = elem->size;
       for (int i = 0; i < ti->ndims; i++) {
          tree_t r = range_of(type, i);
          int64_t low, high;
@@ -43,10 +50,12 @@ static void fill_array_type_info(mir_unit_t *mu, type_t type, type_info_t *ti)
          ti->size *= MAX(high - low + 1, 0);
       }
 
-      ti->type = mir_carray_type(mu, ti->size, elem->type);
+      ti->type = mir_carray_type(mu, ti->size * ti->stride, base->type);
+   }
+   else {
+      ti->type = mir_uarray_type(mu, ti->udims, base->type);
+      ti->size = SIZE_MAX;
    }
-   else
-      ti->type = mir_uarray_type(mu, ti->ndims, elem->type);
 
    ti->stamp = elem->stamp;
 }
@@ -58,12 +67,14 @@ const type_info_t *type_info(mir_unit_t *mu, type_t type)
       return ti;
 
    ti = mir_malloc(mu, sizeof(type_info_t));
-   ti->type  = MIR_NULL_TYPE;
-   ti->stamp = MIR_NULL_STAMP;
-   ti->kind  = type_base_kind(type);
-   ti->ndims = 0;
-   ti->udims = 0;
-   ti->size  = 1;
+   ti->source = type;
+   ti->type   = MIR_NULL_TYPE;
+   ti->stamp  = MIR_NULL_STAMP;
+   ti->kind   = type_base_kind(type);
+   ti->ndims  = 0;
+   ti->udims  = 0;
+   ti->size   = 1;
+   ti->stride = 1;
 
    switch (type_kind(type)) {
    case T_SUBTYPE:
@@ -128,7 +139,11 @@ const type_info_t *type_info(mir_unit_t *mu, type_t type)
       break;
 
    case T_ACCESS:
-      should_not_reach_here();
+      {
+         mir_type_t designated = type_info(mu, type_designated(type))->type;
+         ti->type = mir_access_type(mu, designated);
+      }
+      break;
 
    case T_REAL:
       {
@@ -150,6 +165,76 @@ const type_info_t *type_info(mir_unit_t *mu, type_t type)
       fatal_trace("cannot lower type kind %s", type_kind_str(type_kind(type)));
    }
 
+#ifdef DEBUG
+   LOCAL_TEXT_BUF tb = tb_new();
+   if (is_anonymous_subtype(type))
+      tb_cat(tb, "Anonymous subtype ");
+   tb_printf(tb, "%s %s", type_pp(type), type_kind_str(ti->kind));
+
+   if (ti->kind == T_ARRAY) {
+      tb_printf(tb, " ndims:%d udims:%d", ti->ndims, ti->udims);
+      if (ti->size == SIZE_MAX)
+         tb_cat(tb, " size:*");
+      else
+         tb_printf(tb, " size:%zd", ti->size);
+      if (ti->stride == SIZE_MAX)
+         tb_cat(tb, " stride:*");
+      else
+         tb_printf(tb, " stride:%zd", ti->stride);
+   }
+
+   mir_comment(mu, "%s", tb_get(tb));
+#endif
+
    mir_put_priv(mu, type, ti);
    return ti;
 }
+
+#if 0
+mir_value_t lower_array_stride(mir_unit_t *mu, const type_info_t *ti,
+                               mir_value_t array)
+{
+   mir_type_t t_offset = mir_offset_type(mu);
+
+   if (ti->stride < SIZE_MAX)
+      return mir_const(mu, t_offset, ti->stride);
+
+   assert(mir_is(mu, array, MIR_TYPE_UARRAY));
+
+   int pos = ti->ndims;
+   mir_value_t stride = mir_const(mu, t_offset, 1);
+
+   for (const type_info_t *elem = type_info(mu, type_elem(ti->source));
+        elem->kind == T_ARRAY; elem = type_info(mu, type_elem(elem->source))) {
+
+      if (elem->size < SIZE_MAX) {
+         mir_value_t size = mir_const(mu, t_offset, elem->size);
+         stride = mir_build_mul(mu, t_offset, stride, size);
+         break;
+      }
+
+      mir_value_t length = mir_build_uarray_len(mu, array, pos++);
+      for (int i = 1; i < elem->ndims; i++) {
+         mir_value_t dim_len = mir_build_uarray_len(mu, array, pos++);
+         length = mir_build_mul(mu, t_offset, dim_len, length);
+      }
+   }
+
+   assert(pos == ti->udims);
+   return stride;
+}
+
+mir_value_t lower_total_elements(mir_unit_t *mu, const type_info_t *ti,
+                                 mir_value_t array)
+{
+   mir_type_t t_offset = mir_offset_type(mu);
+
+   if (ti->size < SIZE_MAX && ti->stride < SIZE_MAX)
+      return mir_const(mu, t_offset, ti->size * ti->stride);
+
+   mir_value_t total = lower_array_length(mu, ti, array);
+   mir_value_t stride = lower_array_stride(mu, ti, array);
+
+   return mir_build_mul(mu, t_offset, total, stride);
+}
+#endif
