@@ -86,7 +86,7 @@ static const imask_t has_map[V_LAST_NODE_KIND] = {
    (I_LEFT | I_RIGHT | I_SUBKIND),
 
    // V_BASSIGN
-   (I_TARGET | I_VALUE | I_SUBKIND | I_DELAY),
+   (I_TARGET | I_VALUE | I_DELAY),
 
    // V_UNARY
    (I_VALUE | I_SUBKIND),
@@ -131,7 +131,7 @@ static const imask_t has_map[V_LAST_NODE_KIND] = {
    (I_IDENT | I_TYPE | I_RANGES | I_DECLS),
 
    // V_ENUM_NAME
-   (I_IDENT | I_TYPE),
+   (I_IDENT | I_TYPE | I_VALUE),
 
    // V_UNION_DECL
    (I_IDENT | I_DECLS),
@@ -155,10 +155,10 @@ static const imask_t has_map[V_LAST_NODE_KIND] = {
    (I_VALUE | I_STMTS),
 
    // V_TASK_DECL
-   (I_IDENT | I_STMTS | I_DECLS | I_PORTS),
+   (I_IDENT | I_IDENT2 | I_STMTS | I_DECLS | I_PORTS),
 
    // V_FUNC_DECL
-   (I_IDENT | I_STMTS | I_DECLS | I_TYPE | I_PORTS),
+   (I_IDENT | I_IDENT2 | I_STMTS | I_DECLS | I_TYPE | I_PORTS),
 
    // V_WAIT
    (I_VALUE | I_STMTS),
@@ -245,10 +245,43 @@ static const imask_t has_map[V_LAST_NODE_KIND] = {
    (I_VALUE),
 
    // V_GENVAR_DECL
-   (I_IDENT),
+   (I_IDENT | I_TYPE),
 
    // V_FOR_GENERATE
    (I_LEFT | I_VALUE | I_RIGHT | I_STMTS),
+
+   // V_DEASSIGN
+   (I_TARGET),
+
+   // V_FORCE
+   (I_TARGET | I_VALUE),
+
+   // V_RELEASE
+   (I_TARGET | I_VALUE),
+
+   // V_DISABLE
+   (I_IDENT),
+
+   // V_HIER_REF
+   (I_IDENT | I_IDENT2 | I_REF),
+
+   // V_TF_PORT_DECL
+   (I_IDENT | I_SUBKIND | I_RANGES | I_TYPE | I_VALUE),
+
+   // V_RETURN
+   (I_REF | I_VALUE),
+
+   // V_OP_ASSIGN
+   (I_TARGET | I_SUBKIND | I_VALUE),
+
+   // V_STRUCT_REF
+   (I_VALUE | I_IDENT | I_REF),
+
+   // V_PACKAGE
+   (I_DECLS | I_IDENT),
+
+   // V_MIN_TYP_MAX
+   (I_LEFT | I_VALUE | I_RIGHT)
 };
 
 static const char *kind_text_map[V_LAST_NODE_KIND] = {
@@ -270,7 +303,10 @@ static const char *kind_text_map[V_LAST_NODE_KIND] = {
    "V_INST_BODY",     "V_PORT_CONN",   "V_PART_SELECT",   "V_IF_GENERATE",
    "V_EVENT_TRIGGER", "V_USER_FCALL",  "V_UDP_LEVEL",     "V_UDP_EDGE",
    "V_SPECPARAM",     "V_FORK",        "V_ATTR_INST",     "V_USER_TCALL",
-   "V_VOID_CALL",     "V_GENVAR_DECL", "V_FOR_GENERATE",
+   "V_VOID_CALL",     "V_GENVAR_DECL", "V_FOR_GENERATE",  "V_DEASSIGN",
+   "V_FORCE",         "V_RELEASE",     "V_DISABLE",       "V_HIER_REF",
+   "V_TF_PORT_DECL",  "V_RETURN",      "V_OP_ASSIGN",     "V_STRUCT_REF",
+   "V_PACKAGE",       "V_MIN_TYP_MAX"
 };
 
 static const change_allowed_t change_allowed[] = {
@@ -344,6 +380,11 @@ ident_t vlog_ident2(vlog_node_t v)
    return item->ident;
 }
 
+bool vlog_has_ident2(vlog_node_t v)
+{
+   return lookup_item(&vlog_object, v, I_IDENT2)->ident != NULL;
+}
+
 void vlog_set_ident2(vlog_node_t v, ident_t i)
 {
    lookup_item(&vlog_object, v, I_IDENT2)->ident = i;
@@ -414,7 +455,8 @@ vlog_node_t vlog_port(vlog_node_t v, unsigned n)
 void vlog_add_port(vlog_node_t v, vlog_node_t p)
 {
    assert(p != NULL);
-   assert(p->object.kind == V_REF || p->object.kind == V_PORT_DECL);
+   assert(p->object.kind == V_REF || p->object.kind == V_PORT_DECL
+          || p->object.kind == V_TF_PORT_DECL);
    vlog_array_add(lookup_item(&vlog_object, v, I_PORTS), p);
    object_write_barrier(&(v->object), &(p->object));
 }
