@@ -29,7 +29,10 @@ void vlog_trans(vlog_node_t mod, tree_t out);
 void vlog_lower_deferred(mir_unit_t *mu, object_t *obj);
 void vlog_lower_udp(mir_unit_t *mu, object_t *obj);
 void vlog_lower_block(mir_context_t *mc, ident_t parent, tree_t b);
+mir_unit_t *vlog_lower_thunk(mir_context_t *mc, ident_t parent, vlog_node_t v);
 vlog_node_t vlog_new_instance(vlog_node_t mod, vlog_node_t inst,
                               ident_t prefix);
+vlog_node_t vlog_generate_instance(vlog_node_t v, vlog_node_t genvar,
+                                   int32_t value, ident_t prefix);
 
 #endif  // _VLOG_PHASE_H
