@@ -22,13 +22,17 @@
 
 bool vlog_is_net(vlog_node_t v);
 unsigned vlog_dimensions(vlog_node_t v);
-int64_t vlog_get_const(vlog_node_t v);
+bool vlog_get_const(vlog_node_t v, int64_t *value);
 bool vlog_is_const(vlog_node_t v);
-void vlog_bounds(vlog_node_t v, int64_t *left, int64_t *right);
+bool vlog_is_up(vlog_node_t v);
+bool vlog_bounds(vlog_node_t v, int64_t *left, int64_t *right);
 unsigned vlog_size(vlog_node_t v);
 bool is_top_level(vlog_node_t v);
 bool is_data_type(vlog_node_t v);
 vlog_node_t vlog_longest_static_prefix(vlog_node_t v);
+bool vlog_equal_node(vlog_node_t a, vlog_node_t b);
+uint32_t vlog_hash_node(vlog_node_t v);
+vlog_node_t vlog_get_type(vlog_node_t v);
 
 #define CANNOT_HANDLE(v) do {                                           \
       fatal_at(vlog_loc(v), "cannot handle %s in %s" ,                  \
