@@ -84,6 +84,14 @@ typedef struct {
 
 DEF_CLASS(module, vpiModule, scope.object);
 
+typedef struct {
+   c_abstractScope  scope;
+   tree_t           block;
+   rt_scope_t      *rtscope;
+} c_genScope;
+
+DEF_CLASS(genScope, vpiGenScope, scope.object);
+
 typedef struct {
    c_vpiObject      object;
    vpiObjectList    args;
@@ -581,6 +589,8 @@ static c_vpiObject *build_expr(vlog_node_t v, c_abstractScope *scope)
          vlog_node_t d = vlog_ref(v);
          if (vlog_kind(d) == V_PORT_DECL)
             d = vlog_ref(d);
+         else if (vlog_kind(d) == V_TF_PORT_DECL || vlog_kind(d) == V_FUNC_DECL)
+            return &(build_operation(v)->expr.object);  /// XXX: hack
 
          vpiObjectList *list =
             expand_lazy_list(&(scope->object), &(scope->decls));
@@ -600,6 +610,8 @@ static c_vpiObject *build_expr(vlog_node_t v, c_abstractScope *scope)
    case V_EMPTY:
    case V_PREFIX:
    case V_POSTFIX:
+   case V_COND_EXPR:
+   case V_PART_SELECT:
    case V_BIT_SELECT:   // XXX: check this
       return &(build_operation(v)->expr.object);
    default:
@@ -654,27 +666,46 @@ static c_module *build_module(vlog_node_t v, tree_t block, rt_scope_t *s)
    return m;
 }
 
-static c_module *cached_module(tree_t block, rt_scope_t *s)
+static c_genScope *build_genScope(vlog_node_t v, tree_t block, rt_scope_t *s)
+{
+   c_genScope *m = new_object(sizeof(c_genScope), vpiGenScope);
+   init_abstractScope(&m->scope, v);
+   m->block = block;
+   m->rtscope = s;
+
+   return m;
+}
+
+static c_abstractScope *cached_scope(tree_t block, rt_scope_t *s)
 {
    assert(tree_kind(block) == T_BLOCK);
 
    hash_t *cache = vpi_context()->objcache;
-   c_module *m = hash_get(cache, block);
-   if (m == NULL) {
+   c_abstractScope *as = hash_get(cache, block);
+   if (as == NULL) {
       tree_t hier = tree_decl(block, 0);
       assert(tree_kind(hier) == T_HIER);
 
       tree_t wrap = tree_ref(hier);
       assert(tree_kind(wrap) == T_VERILOG);
 
-      vlog_node_t body = tree_vlog(wrap);
-      assert(vlog_kind(body) == V_INST_BODY);
+      vlog_node_t v = tree_vlog(wrap);
 
-      m = build_module(body, block, s);
-      hash_put(cache, block, m);
+      switch (vlog_kind(v)) {
+      case V_INST_BODY:
+         as = &(build_module(v, block, s)->scope);
+         break;
+      case V_BLOCK:
+         as = &(build_genScope(v, block, s)->scope);
+         break;
+      default:
+         should_not_reach_here();
+      }
+
+      hash_put(cache, block, as);
    }
 
-   return m;
+   return as;
 }
 
 static jit_t *vpi_get_jit(vpi_context_t *c)
@@ -931,7 +962,12 @@ void vpi_get_value(vpiHandle handle, p_vpi_value value_p)
       case vpiBinaryConst:
          {
             number_t n = vlog_number(con->expr.where);
-            vpi_format_number(n, value_p->format, c->valuestr);
+
+            const uint64_t *abits, *bbits;
+            number_get(n, &abits, &bbits);
+
+            vpi_format_number(number_width(n), abits, bbits, value_p->format,
+                              c->valuestr);
 
             value_p->value.str = (PLI_BYTE8 *)tb_get(c->valuestr);
             return;
@@ -944,10 +980,10 @@ void vpi_get_value(vpiHandle handle, p_vpi_value value_p)
       int size = c->args[op->argslot].integer;
       assert(size <= 64);
 
-      uint64_t abits = c->args[op->argslot + 1].integer;
-      uint64_t bbits = c->args[op->argslot + 2].integer;
+      uint64_t abits[1] = { c->args[op->argslot + 1].integer };
+      uint64_t bbits[1] = { c->args[op->argslot + 2].integer };
 
-      vpi_format_number2(size, abits, bbits, value_p->format, c->valuestr);
+      vpi_format_number(size, abits, bbits, value_p->format, c->valuestr);
 
       value_p->value.str = (PLI_BYTE8 *)tb_get(c->valuestr);
       return;
@@ -957,9 +993,24 @@ void vpi_get_value(vpiHandle handle, p_vpi_value value_p)
    if (decl != NULL) {
       sig_shared_t **ss = vpi_get_ptr(decl);
       rt_signal_t *s = container_of(*ss, rt_signal_t, shared);
-      number_t num = number_pack(signal_value(s), signal_width(s));
 
-      vpi_format_number(num, value_p->format, c->valuestr);
+      const int width = signal_width(s);
+      const int nwords = (width + 63) / 64;
+      uint64_t *mem LOCAL = xmalloc_array(nwords * 2, sizeof(uint64_t));
+      uint64_t *abits = mem, *bbits = mem + nwords;
+
+      for (int i = 0; i < nwords * 2; i++)
+         mem[i] = 0;
+
+      const uint8_t *vals = signal_value(s);
+
+      for (int i = 0; i < width; i++) {
+         const int pos = width - 1 - i;
+         abits[pos / 64] |= (uint64_t)(vals[i] & 1) << (pos % 64);
+         bbits[pos / 64] |= (uint64_t)((vals[i] >> 1) & 1) << (pos % 64);
+      }
+
+      vpi_format_number(width, abits, bbits, value_p->format, c->valuestr);
 
       value_p->value.str = (PLI_BYTE8 *)tb_get(c->valuestr);
       return;
@@ -984,14 +1035,28 @@ vpiHandle vpi_put_value(vpiHandle handle, p_vpi_value value_p,
 
    c_sysFuncCall *fcall = is_sysFuncCall(obj);
    if (fcall != NULL && c->args != NULL) {
-      assert(fcall->systfcall.callback->systf.sysfunctype == vpiTimeFunc);
-      assert(value_p->format == vpiTimeVal);
+      switch (fcall->systfcall.callback->systf.sysfunctype) {
+      case vpiTimeFunc:
+         {
+            assert(value_p->format == vpiTimeVal);
 
-      const p_vpi_time tm = value_p->value.time;
-      c->args[0].integer = (uint64_t)tm->high << 32 | tm->low;
-      c->args[1].integer = 0;
+            const p_vpi_time tm = value_p->value.time;
+            c->args[0].integer = (uint64_t)tm->high << 32 | tm->low;
+            c->args[1].integer = 0;
 
-      return NULL;
+            return NULL;
+         }
+
+      case vpiIntFunc:
+         {
+            assert(value_p->format == vpiIntVal);
+
+            c->args[0].integer = value_p->value.integer;
+            c->args[1].integer = 0;
+
+            return NULL;
+         }
+      }
    }
 
    vpi_error(vpiError, &(obj->loc), "cannot change value of %s",
@@ -1128,8 +1193,9 @@ vpiHandle vpi_bind_foreign(ident_t name, vlog_node_t where)
    }
 
    rt_model_t *m = get_model();
-   rt_scope_t *scope = get_active_scope(m);
-   c_module *mod = cached_module(scope->where, scope);
+   rt_scope_t *rs = get_active_scope(m);
+
+   c_abstractScope *scope = cached_scope(rs->where, rs);
 
    for (int i = 0; i < c->systasks.count; i++) {
       c_vpiObject *obj = from_handle(c->systasks.items[i]);
@@ -1144,11 +1210,14 @@ vpiHandle vpi_bind_foreign(ident_t name, vlog_node_t where)
 
       c_sysTfCall *call;
       if (cb->systf.type == vpiSysTask)
-         call = &(build_sysTaskCall(where, cb, &(mod->scope))->systfcall);
+         call = &(build_sysTaskCall(where, cb, scope)->systfcall);
       else
-         call = &(build_sysFuncCall(where, cb, &(mod->scope))->systfcall);
+         call = &(build_sysFuncCall(where, cb, scope)->systfcall);
 
-      return (call->handle = internal_handle_for(&(call->tfcall.object)));
+      call->handle = internal_handle_for(&(call->tfcall.object));
+
+      hash_put(c->objcache, where, call);
+      return call->handle;
    }
 
    return NULL;
