@@ -101,6 +101,7 @@
 #define F_NOTBSD  (1 << 25)
 #define F_ARRAYS  (1 << 26)
 #define F_SEED    (1 << 27)
+#define F_PERFILE (1 << 28)
 
 typedef struct test test_t;
 typedef struct param param_t;
@@ -155,6 +156,10 @@ static char bin_dir[PATH_MAX];
 static bool is_tty = false;
 static bool force_jit = false;
 static bool force_precompile = false;
+static bool capture_xml = false;
+static bool capture_html = false;
+static bool diff_html = false;
+static bool check_html = false;
 
 #ifdef __MINGW32__
 static char *strndup(const char *s, size_t n)
@@ -432,6 +437,8 @@ static bool parse_test_list(void)
             test->flags |= F_TCL;
          else if (strcmp(opt, "shuffle") == 0)
             test->flags |= F_SHUFFLE;
+         else if (strcmp(opt, "per-file") == 0)
+            test->flags |= F_PERFILE;
          else if (strcmp(opt, "no-collapse") == 0)
             test->flags |= F_NOCOLL;
          else if (strcmp(opt, "dump-arrays") == 0)
@@ -917,9 +924,14 @@ static bool run_test(test_t *test)
          push_arg(&args, "%s" DIR_SEP "regress" DIR_SEP "%s.vhd",
                   test_dir, test->name);
 
-      if (test->flags & (F_MIXED | F_VERILOG))
-         push_arg(&args, "%s" DIR_SEP "regress" DIR_SEP "%s.v",
-                  test_dir, test->name);
+      if (test->flags & (F_MIXED | F_VERILOG)) {
+         if (file_exists("%s/regress/%s.sv", test_dir, test->name))
+            push_arg(&args, "%s" DIR_SEP "regress" DIR_SEP "%s.sv",
+                     test_dir, test->name);
+         else
+            push_arg(&args, "%s" DIR_SEP "regress" DIR_SEP "%s.v",
+                     test_dir, test->name);
+      }
 
       if (test->flags & F_RELAX)
          push_arg(&args, "--relax=%s", test->relax);
@@ -1087,25 +1099,36 @@ static bool run_test(test_t *test)
       for (char *p = unit; *p; p++)
          *p = toupper((int)*p);
 
+      char *html_dir;
+      if (capture_html)
+         html_dir = xasprintf("%s/html/%s", cwd, test->name);
+      else
+         html_dir = strdup("./html");
+
+      char *xml_out;
+      if (capture_xml)
+         xml_out = xasprintf("%s/regress/gold/%s.xml", test_dir, test->name);
+      else
+         xml_out = strdup("export.xml");
+
       push_arg(&args, "%s/nvc%s", bin_dir, EXEEXT);
       push_arg(&args, "--cover-report");
-      push_arg(&args, "--output=./html");
+      push_arg(&args, "--output=%s", html_dir);
+      if (test->flags & F_PERFILE)
+         push_arg(&args, "--per-file");
       push_arg(&args, "%s.ncdb", test->name);
       push_arg(&args, "--cover-export");
       push_arg(&args, "--relative=%s" DIR_SEP "regress", test_dir);
-      push_arg(&args, "--output=export.xml");
-      //push_arg(&args, "--out=%s/regress/gold/%s.xml", test_dir, test->name);
+      push_arg(&args, "--out=%s", xml_out);
       push_arg(&args, "--format=xml");
       push_arg(&args, "%s.ncdb", test->name);
 
-      free(unit);
-
       if (run_cmd(outf, &args) != RUN_OK) {
          failed("coverage report");
          result = false;
          goto out_print;
       }
-      else if (!file_exists("html/index.html")) {
+      else if (!file_exists("%s/index.html", html_dir)) {
          failed("missing coverage report index.html");
          result = false;
          goto out_print;
@@ -1117,13 +1140,49 @@ static bool run_test(test_t *test)
 #endif
       push_arg(&args, "-u");
       push_arg(&args, "%s/regress/gold/%s.xml", test_dir, test->name);
-      push_arg(&args, "export.xml");
+      push_arg(&args, "%s", xml_out);
 
       if (run_cmd(outf, &args) != RUN_OK) {
          failed("XML mismatch");
          result = false;
          goto out_print;
       }
+
+      if (diff_html) {
+         char *cmp_dir = xasprintf("%s/html/%s", cwd, test->name);
+
+         push_arg(&args, "%s", DIFF_PATH);
+#if defined __MINGW32__ || defined __CYGWIN__
+         push_arg(&args, "--strip-trailing-cr");
+#endif
+         push_arg(&args, "-ru");
+         push_arg(&args, "%s", cmp_dir);
+         push_arg(&args, "%s", html_dir);
+
+         if (run_cmd(outf, &args) != RUN_OK) {
+            failed("HTML mismatch");
+            result = false;
+            goto out_print;
+         }
+
+         free(cmp_dir);
+      }
+
+      if (check_html) {
+         push_arg(&args, "/usr/bin/env");
+         push_arg(&args, "vnu");
+         push_arg(&args, "%s", html_dir);
+
+         if (run_cmd(outf, &args) != RUN_OK) {
+            failed("HTML validation errors");
+            result = false;
+            goto out_print;
+         }
+      }
+
+      free(html_dir);
+      free(xml_out);
+      free(unit);
    }
 
    if (test->flags & F_WAVE) {
@@ -1340,9 +1399,18 @@ int main(int argc, char **argv)
    if (getenv("QUICK"))
       return 0;
 
+   static struct option long_options[] = {
+      { "capture-html", no_argument, 0, 'h' },
+      { "diff-html",    no_argument, 0, 'H' },
+      { "check-html",   no_argument, 0, 'C' },
+      { "capture-xml",  no_argument, 0, 'x' },
+      { 0, 0, 0, 0 }
+   };
+
    bool print_stats= false;
-   int c;
-   while ((c = getopt(argc, argv, "sjp")) != -1) {
+   int c, index = 0;
+   const char *spec = ":sjp";
+   while ((c = getopt_long(argc, argv, spec, long_options, &index)) != -1) {
       switch (c) {
       case 's':
          print_stats = true;
@@ -1353,7 +1421,24 @@ int main(int argc, char **argv)
       case 'p':
          force_precompile = true;
          break;
+      case 'h':
+         capture_html = true;
+         make_dir("html");
+         break;
+      case 'x':
+         capture_xml = true;
+         break;
+      case 'H':
+         diff_html = true;
+         break;
+      case 'C':
+         check_html = true;
+         break;
       case '?':
+         if (optopt == 0)
+            fprintf(stderr, "Invalid option %s\n", argv[optind - 1]);
+         else
+            fprintf(stderr, "Invalid option -%c\n", optopt);
          return EXIT_FAILURE;
       default:
          abort();
