@@ -93,9 +93,9 @@ START_TEST(test_comp)
    input_from_file(TESTDIR "/elab/comp.vhd");
 
    const error_t expect[] = {
-      { 77, "port X in entity WORK.E2 without a default value has no "
+      { 79, "port X in entity WORK.E2 without a default value has no "
         "corresponding port in component E2" },
-      { 83, "port X in component E3 has type BIT which is incompatible "
+      { 85, "port X in component E3 has type BIT which is incompatible "
         "with type INTEGER in entity WORK.E3" },
       { -1, NULL }
    };
@@ -1440,10 +1440,12 @@ START_TEST(test_mixed1)
       { 48, "cannot connect VHDL signal with type BIT to Verilog output "
         "port 'three'" },
       { 56, "missing matching VHDL generic declaration for Verilog parameter "
-        "'g1' in component MOD1" },
+        "'g1' with no default value in component MOD1" },
       { 67, "generic G1 should have type INTEGER to match corresponding "
         "Verilog parameter" },
       { 79, "generic G2 not found in Verilog module mod1" },
+      { 93, "VHDL port direction IN does not match corresponding Verilog "
+        "port 'three' which is declared as output" },
       { -1, NULL }
    };
    expect_errors(expect);
@@ -1612,6 +1614,8 @@ START_TEST(test_body1)
    expect_errors(expect);
 
    (void)run_elab();
+
+   check_expected_errors();
 }
 END_TEST
 
@@ -1626,8 +1630,8 @@ START_TEST(test_issue759)
 
    mir_context_t *mc = get_mir();
    unit_registry_t *ur = get_registry();
-   jit_t *jit = jit_new(ur, mc);
    cover_data_t *cover = cover_data_init(COVER_MASK_ALL, 0, 0);
+   jit_t *jit = jit_new(ur, mc, cover);
 
    rt_model_t *m = model_new(jit, cover);
    tree_t e = elab(tree_to_object(a), jit, ur, mc, cover, NULL, m);
@@ -1741,12 +1745,16 @@ END_TEST
 START_TEST(test_vlog1)
 {
    const error_t expect[] = {
-      { 11, "name of Verilog module sub1 in library unit WORK.SUB1 does "
+      { 12, "name of Verilog module sub1 in library unit WORK.SUB1 does "
         "not match name SUB1 in module instance u2" },
-      { 12, "expected 2 port connections for module sub1 but found 1" },
-      { 13, "module bad not found in library WORK" },
-      { 14, "missing value for parameter p" },
-      { 15, "module sub1 has 1 parameter but 2 values given" },
+      { 13, "expected 2 port connections for module sub1 but found 1" },
+      { 14, "module bad not found in library WORK" },
+      { 15, "missing value for parameter p" },
+      { 16, "module sub1 has 1 parameter but 2 values given" },
+      { 17, "missing port connection for 'x'" },
+      { 18, "'z' cannot be driven by continuous assignment from port 'y'" },
+      { 19, "expression cannot be driven by continuous assignment from "
+        "port 'y'" },
       { -1, NULL }
    };
    expect_errors(expect);
@@ -1758,7 +1766,7 @@ START_TEST(test_vlog1)
 
    mir_context_t *mc = get_mir();
    unit_registry_t *ur = get_registry();
-   jit_t *j = jit_new(ur, mc);
+   jit_t *j = jit_new(ur, mc, NULL);
    rt_model_t *m = model_new(j, NULL);
 
    tree_t top = elab(obj, j, ur, mc, NULL, NULL, m);
@@ -1947,7 +1955,7 @@ START_TEST(test_issue1012)
 
    mir_context_t *mc = get_mir();
    unit_registry_t *ur = get_registry();
-   jit_t *jit = jit_new(ur, mc);
+   jit_t *jit = jit_new(ur, mc, NULL);
    cover_data_t *cover = cover_data_init(COVER_MASK_TOGGLE, 0, 0);
    rt_model_t *m = model_new(jit, NULL);
 
@@ -2033,6 +2041,80 @@ START_TEST(test_issue1204)
 }
 END_TEST
 
+START_TEST(test_clone1)
+{
+   analyse_file(TESTDIR "/elab/clone1.v", NULL, NULL, NULL);
+
+   input_from_file(TESTDIR "/elab/clone1.vhd");
+
+   tree_t e = run_elab();
+   fail_if(e == NULL);
+
+   tree_t top = tree_stmt(e, 0);
+   fail_unless(tree_kind(top) == T_BLOCK);
+   ck_assert_int_eq(tree_stmts(top), 10);
+
+   tree_t u0 = tree_stmt(tree_stmt(tree_stmt(top, 0), 0), 0);
+   fail_unless(tree_kind(u0) == T_BLOCK);
+
+   tree_t h0 = tree_decl(u0, 0);
+   fail_unless(tree_kind(h0) == T_HIER);
+
+   ident_t dotted0 = ident_new("WORK.TOP.G(1).U.sub");
+   fail_unless(tree_ident(h0) == dotted0);
+   fail_unless(tree_ident2(h0) == dotted0);
+
+   const int ndecls = tree_decls(u0);
+   const int nstmts = tree_stmts(u0);
+
+   for (int i = 1; i < 10; i++) {
+      tree_t u = tree_stmt(tree_stmt(tree_stmt(top, i), 0), 0);
+      fail_unless(tree_kind(u) == T_BLOCK);
+
+      ck_assert_int_eq(tree_decls(u), ndecls);
+      ck_assert_int_eq(tree_stmts(u), nstmts);
+
+      ident_t id = ident_sprintf("WORK.TOP.G(%d).U.sub", i + 1);
+
+      tree_t h = tree_decl(u, 0);
+      fail_unless(tree_kind(h) == T_HIER);
+      fail_unless(tree_ident(h) == id);
+      // fail_unless(tree_ident2(h) == dotted0);  TODO
+
+      for (int j = 1; j < ndecls; j++)
+         ck_assert_ptr_eq(tree_decl(u0, j), tree_decl(u, j));
+
+      // TODO
+      //for (int j = 0; j < nstmts; j++)
+      //   ck_assert_ptr_eq(tree_stmt(u0, j), tree_stmt(u, j));
+   }
+
+   fail_if_errors();
+}
+END_TEST
+
+START_TEST(test_mixed2)
+{
+   analyse_file(TESTDIR "/elab/mixed2.v", NULL, NULL, NULL);
+
+   input_from_file(TESTDIR "/elab/mixed2.vhd");
+
+   const error_t expect[] = {
+      { 12, "actual length 20 does not match formal length 8" },
+      { 22, "actual length 6 does not match formal length 8" },
+      { 33, "actual length 100 does not match formal length 4" },
+      { 44, "actual length 8 does not match formal length 4" },
+      { -1, NULL }
+   };
+   expect_errors(expect);
+
+   tree_t e = run_elab();
+   fail_unless(e == NULL);
+
+   check_expected_errors();
+}
+END_TEST
+
 Suite *get_elab_tests(void)
 {
    Suite *s = suite_create("elab");
@@ -2142,6 +2224,8 @@ Suite *get_elab_tests(void)
    tcase_add_test(tc, test_issue1195);
    tcase_add_test(tc, test_issue1201);
    tcase_add_test(tc, test_issue1204);
+   tcase_add_test(tc, test_clone1);
+   tcase_add_test(tc, test_mixed2);
    suite_add_tcase(s, tc);
 
    return s;
